/*
  layeredbase.h
  Copyright (C) 2008 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

//#define DEBUG

#include "qgmap.h"
#include "layeredbase.h"
#include "layeredwidget.h"
#include <qwidget.h>
#include <qpixmap.h>
#include <stdio.h>

LayeredBase::LayeredBase( QWidget *parent, const char *name, WFlags fl) :
	QWidget(parent, name, fl)
{
	clicked_layer = -1;
	paint_blocked = false;
	paint_request = false;
}

void LayeredBase::addWidget( LayeredWidget *widget )
{
	int i;
	i = array.size();
	array.resize(i+1);
	array[i] = widget;
}

void LayeredBase::paintEvent( QPaintEvent *e UNUSED)
{
	Enter();
	QPixmap scr(this->size());
	uint i;

	for(i=0;i<array.count();i++)
		array[i]->paintEvent(scr);

	bitBlt(this, QPoint(0,0), &scr, scr.rect());
}

void LayeredBase::showEvent( QShowEvent *e UNUSED)
{
	Enter();
	uint i;
	for(i=0; i < array.count(); i++){
		array[i]->showEvent();
	}
}

void LayeredBase::keyPressEvent ( QKeyEvent * e )
{
	int i;

	// default state is ignore
	e->ignore();

	blockPaint();
	for(i = array.count()-1; i >= 0; i--){
		LayeredWidget *w = array[(uint)i];
		if(w->keyPressEvent(e)){
			// set the accept flag, if the key event is accepted.
			e->accept();
			break;
		}
	}
	unblockPaint();
	return;
}

void LayeredBase::keyReleaseEvent ( QKeyEvent * e )
{
	int i;

	blockPaint();
	for(i = array.count()-1; i >= 0; i--){
		LayeredWidget *w = array[(uint)i];
		if(w->keyReleaseEvent(e)){
			break;
		}
	}
	unblockPaint();
	return;
}

void LayeredBase::wheelEvent( QWheelEvent *e )
{
	Enter();
	int i;

	blockPaint();
	for(i = array.count()-1; i >= 0; i--){
		LayeredWidget *w = array[(uint)i];

		if(w->wheelEvent(e))
			break;
	}
	unblockPaint();
	return;
}

void LayeredBase::mousePressEvent( QMouseEvent *e )
{
	Enter();
	int i;

	blockPaint();
	for(i = array.count()-1; i >= 0; i--){
		LayeredWidget *w = array[(uint)i];

		if(w->geometry().contains(e->pos())){
			if(w->mousePressEvent(e)){
				clicked_layer = i;
				break;
			}
		}
	}
	unblockPaint();
}

void LayeredBase::mouseMoveEvent( QMouseEvent *e )
{
	Enter();
	if(clicked_layer<0)
		return;

	array[clicked_layer]->mouseMoveEvent(e);
}

void LayeredBase::mouseReleaseEvent( QMouseEvent *e )
{
	if(clicked_layer < 0)
		return;

	array[clicked_layer]->mouseReleaseEvent(e);
	clicked_layer = -1;
}

void LayeredBase::blockPaint()
{
	paint_blocked = true;
}

void LayeredBase::unblockPaint()
{
	if(paint_request)
		QWidget::repaint(FALSE);
	paint_blocked = false;
}

void LayeredBase::repaint()
{
	if(paint_blocked)
		paint_request = true;
	else
		QWidget::repaint(FALSE);
}
